#include "JJClientObj.h"

#include "JLinkObj.h"
#include "JInteger.h"
#include "JIntegerData.h"
#include "JRealData.h"
#include "JBlockData.h"
#include "JBytesData.h"
#include "JCharacter.h"
#include "JStringData.h"
#include "JStringProperty.h"
#include "JIntegerProperty.h"
#include "JSocketException.h"
#include "JUnknownHostException.h"
#include <stdio.h>
#include <io.h>
#include <winsock2.h>
#include "MindtelSocket.h"

#include <malloc.h>
#include <string.h>
#include <time.h>
char* newstrcpy( char* x ) {
	char* xx = (char*)malloc( strlen( x ) + 1 );
	strcpy( xx, x );
	return( xx );
}



char* theJJClientObj = JJClientObj().Register();		  
void JJClientObj::writeContent(JOutputStream& os) {
  JAddObj::writeContent(os);
  putObject(os, "address", address);
  putInteger(os, "port", port);
  putObject(os, "url", url);
}
int JJClientObj::outputType( int n ) {
	if ( n < MaxValues ) {
		return( JIntegerData::id );
	} else if ( n < MaxValues * 2 ) {
		return( JRealData::id );
	} else {
		return( JStringData::id );
	}
}

void JJClientObj::readContent(JDictionary& dict) {
  JAddObj::readContent(dict);
  JObject *obj = getObject(dict, "address");
  if (obj) address = *(JString*)obj;
  port = getInteger(dict, "port");
  obj = getObject(dict, "url");
  if (obj) url = *(JString*)obj;

}
void JJClientObj::access(int n, JLinkObj& link, const JDataType& data) { 
	if ( n < MaxValues ) {
     INT(data) = intList[n];
	} else if ( n < MaxValues*2 ) {
		DOUBLE(data) = (double)realList[n-MaxValues];
	} else {
		JSTRING(data) = stringList[n-MaxValues*2];
	}
}

const char* JJClientObj::className() const { return "JJClientObj";}
JObject* JJClientObj::clone() const { 
    return new JJClientObj(*this);
}

JJClientObj::JJClientObj() {
  address = "localhost";
  url= "/";
  port = 80;
  readInput = false;
  debug = false;
  igm = 2;
  ogm = 30;
  madeConnection = false;
   for ( int i = 0; i < MaxValues; i++ ) {
	   intList[i] = 0;
	   realList[i] = 0.0;
	   stringList[i] = JString();
   }


}

JJClientObj::JJClientObj(const JJClientObj& obj) { *this=obj;}
JJClientObj::~JJClientObj() { }


/**
 *  Module characteristics associated with inputs.
 */

/**
 *  inputType
 *  
 *  Purpose:  
 *     get the input type corresponding to location
 *     All input types are int
 */

int JJClientObj::inputType(int n) { 
	if ( n == 0 ) {
		return( JStringData::id );
	} else {
		return JIntegerData::id;
	}
}


/**
 *  inputTag
 *
 *  Purpose:
 *     provide a JString describing an input.
 */

JString JJClientObj::inputTag(int n) {
  char* tag[] = {
   // left inputs
   "url",

    // top inputs
    "readInput",  
  };
  return( tag[n] );
 }



boolean JJClientObj::inputAllowed(int n) { 
  return !inputSet(n).last();
}


JFRect JJClientObj::inputArea(int n) {
  if ( n == 0 ) {
    return leftArea( n, 0, 1, 0.25 );
  } else {
    return topArea( n, 0, 1, 0.25 );
  }
}


JFRect JJClientObj::outputArea(int n) {
  return rightArea(n, 0, MaxValues*3, 0.1, 0.8);
}


int JJClientObj::inputFace(int n) {
  if ( n == 0 ) {
    return( LEFT );
  } else {
    return( TOP );
  }
}




JFPoint JJClientObj::inputPoint(int n, JLinkObj& link) {
  if ( n == 0 ) {
    return leftPoint(n, link, 0, 1 );
  } else {
    return topPoint(n, link, 0, 1 );
  }
}


JFPoint JJClientObj::outputPoint(int n, JLinkObj& link) {
  return rightPoint(n, link, 0, MaxValues*3, 0.1, 0.8);
}

/**
 *  Module characteristics associated with outputs
 */



void JJClientObj::paint(JGraphics g, double dx, double dy, JRegion& rgn, double scale) {
  JAddObj::paint(g, dx, dy, rgn, scale);
  JRect rect = getIExtent(dx, dy, scale);
  int kw = depth2*3;
  if ((rect.width > kw) && (rect.height > kw)) {
    if (madeConnection) g.setJColor(JColor::green);
    else g.setJColor(JColor::red);
    g.fillRect(rect.x+depth2, rect.y+depth2, depth2, depth2);
  }
}

void JJClientObj::draw(JGraphics g, int x, int y, int w, int h) {
  int dh = h/3;
  int dy = h/12;
  JRect rect1(x, y+dy, w, dh);
  JRect rect2(x, y+h/2, w, dh);
  drawText(g, "JJClient", rect1);
  drawText(g, address+":"+JInteger::toJString(port), rect2);
}




/**
 *  engine
 *
 *  Purpose:
 *     The main processing engine for this module, called whenever
 *     data is received on an incoming visual link.
 */
bool doit = false;
bool threadRunning = false;
void JJClientObj::engine(int n, JLinkObj& link) {
  switch (n) {
  case URL:
    link.access(JStringData(urlargs));
    break;
  case READINPUT:
    link.access(JIntegerData(readInput));
    if (readInput) {
		if ( !threadRunning ) {
			thread = JThread(this);
			thread.start();
			thread.setPriority(JThread::TNORM_PRIORITY+1);
			threadRunning = true;
		}
		if ( debug ) JComponent::debug( "set doit to true" );
		doit = true;
    }
    break;
  default:
    break;
  }
}

static char* skipPast( char cval, char* scanner ) {
	while ( *scanner ) {
		if ( *scanner == cval ) {
			return( scanner + 1 );
		}
		scanner++;
	}
	//JComponent::debug( "at end!"  );
	return( scanner );
}

void JJClientObj::setupResults( char* buf ) {
	//JComponent::debug( "setupResults entered" );
if ( debug ) JComponent::debug( newstrcpy( buf ));
	//JComponent::debug( "set up ints" );
	for ( int i = 0; i < 10; i++ ) {
		intList[i] = atoi(buf);
		broadcast( i );
		buf = skipPast( ',', buf );
	}
	//JComponent::debug( "set up reals" );
	for ( i = 0; i < 10; i++ ) {
		realList[i] = atof(buf);
		broadcast( i + 10 );
		buf = skipPast( ',', buf );
	}
	//JComponent::debug( "set up strings" );
	for ( i = 0; i < 10; i++ ) {
		stringList[i] = JString( newstrcpy( buf ));
		broadcast( i + 20 );
		buf = skipPast( ',', buf );
	}
	//JComponent::debug( "set up is complete" );
}

void JJClientObj::openSocket() {
  boolean isIPAddr = true;
  for (int i=0; address[i]; i++) {
      if (JCharacter::isAlpha(address[i])) {
        isIPAddr = false;
	  }
  }
  if (isIPAddr) {
    JInetAddress inet = JInetAddress(JInetAddress::getAddrByJString(address));
    socket = JSocket(inet, port);
  } else {
    JInetAddress inet = JInetAddress(address);
    socket = JSocket(inet, port);
  }
}
void JJClientObj::run() {
	if ( debug ) JComponent::debug( "Running thread" );
	try {
		while (1) {
			while (1) {
				if (doit) break;
				//JComponent::debug( "Thread is sleeping" );
				JThread::sleep(1000);
			}
			if ( debug ) JComponent::debug( "Idle loop is finished" );
			if ( doit ) {
				if ( debug ) JComponent::debug( "opening socket" );
				openSocket();
				if ( debug ) JComponent::debug( "Trying to connect" );
				if (socket.connect()) {
					if ( debug ) JComponent::debug( "Successfully connected" );
					JSocketInputStream sis = JSocketInputStream(socket);
					JSocketOutputStream sos = JSocketOutputStream(socket);
					int outFd = sos;					madeConnection = true;
					JString getStr( url + "?" + urlargs );
					if ( urlargs.length() == 0 ) {
						getStr = url;
					}

					char outBuf[500];
					sprintf( outBuf, "GET %s HTTP/1.1\r\nHost: %s\r\n\r\n", (char*)getStr, (char*)address );
if ( debug ) JComponent::debug( newstrcpy( outBuf ));
					int result = send( outFd, outBuf, strlen( outBuf ), 0 );
					char buf[500];
					int len = recv( outFd, buf, 499, 0 ); // recv a little, but just ignore it
					if ( len > 0 ) {
						buf[len] = 0;
						if ( debug ) JComponent::debug( newstrcpy( buf ));
//JComponent::debug( newstrcpy( buf ));
						if ( len > 100 ) {
							if ( debug ) JComponent::debug( newstrcpy( buf + 100 ));
						}
						if ( len > 200 ) {
							if ( debug ) JComponent::debug( newstrcpy( buf + 200 ));
						}
						if ( len > 300 ) {
							if ( debug ) JComponent::debug( newstrcpy( buf + 300 ));
						}
						if ( len > 400 ) {
							if ( debug ) JComponent::debug( newstrcpy( buf + 400 ));
						}
						if ( len > 500 ) {
							if ( debug ) JComponent::debug( newstrcpy( buf + 500 ));
						}
						setupResults( buf );
					}
					if ( debug ) JComponent::debug( "finished with recv" );
				} else {
					if ( debug ) JComponent::debug( "did not connect" );
					madeConnection = false;
				}
				doit = false;
			}
		}
	} catch (JException *e) {
	}
	if ( debug ) JComponent::debug( "Stopped running thread" );
}


JArray JJClientObj::getProperties() {
  JArray properties = JAddObj::getProperties();
  properties.append(JStringProperty("hostName/IPaddress", address));
  properties.append(JIntegerProperty("port", port, 80, 10000));
  properties.append(JStringProperty("url", url));
   properties.append(JIntegerProperty("debug", debug, 0, 1));
 return properties;
}

boolean JJClientObj::updateProperty(JProperty& prop) {
  if (JAddObj::updateProperty(prop)) return true;
  boolean changed = false;
  if (prop.getName() == JString("port")) {
    port = ((JIntegerProperty*)&prop)->value;
    changed = true;
  } else if (prop.getName() == JString("hostName/IPaddress")) {
    address = ((JStringProperty*)&prop)->value;
    changed = true;
  } else if (prop.getName() == JString("url")) {
    url = ((JStringProperty*)&prop)->value;
    changed = true;
  } else if (prop.getName() == JString("debug")) {
	  debug = ((JIntegerProperty*)&prop)->value;
	  changed = true;
  }
  if (changed) repaint();
  return changed;
}









